/* ggisyna - program to display sound graphically
   Copyright (C) 1997       Paul Francis Harrison
   Copyright (C) 1997-1998  Marcus Sundberg

  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the
  Free Software Foundation; either version 2 of the License, or (at your
  option) any later version.

  This program is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  General Public License for more details.

  You should have received a copy of the GNU General Public License along
  with this program; if not, write to the Free Software Foundation, Inc.,
  675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <math.h>
#include <fcntl.h>
#include <vga.h>
#include <vgamouse.h>
typedef unsigned int uint32;

#include "syna.h"

#define SYMBOLCOUNT 5
#define SYMBOLSPACING 40
#define SYMBOLOFFSET 60

static int mouseX = 100, mouseY = 100;
static int fadeCount = 0;
static int track=1, trackCount=1, trackStep=1;
static int symbol=-1, playing=0, showTracks=0;
static uint32 *fb;

unsigned char *backbuf;
int paused=0, bpp=0;


void
gfxOpen(void)
{
	attempt(vga_init(),"initializing svgalib");
	setuid(getuid());
	if (!vga_hasmode(G320x200x256)) 
		error("requesting 320x200 graphics mode");
  
	attempt(vga_setmode(G320x200x256),"entering 320x200 graphics mode");
	//attempt(vga_setmode(G640x480x256),"entering 640x480 graphics mode");
	fb = (uint32 *) vga_getgraphmem();
	bpp=1;
    
	attemptNoDie(mouse_init("/dev/mouse", vga_getmousetype(),
				MOUSE_DEFAULTSAMPLERATE),"initializing mouse");
	mouse_setxrange(0,320-33);
	mouse_setyrange(0,200-33);
	mouse_setposition(160,34);
    
	backbuf = malloc(width*height*bpp*2);
}


void
gfxUpdate(void)
{
    int i = 0;
    uint32 *src = (uint32*)backbuf;
    int pixels = width*height/sizeof(uint32);
    
#define HIGH4MASK 0x0f0f0f0f
#define LOW4MASK  0xf0f0f0f0
    do {
        fb[i] = (src[i]&LOW4MASK)|((src[i+pixels]>>4)&HIGH4MASK);
    } while (++i != pixels);
}


void
gfxClose(void)
{
    mouse_close();
    vga_setmode(TEXT);
}


void
setPalette(int i,int r,int g,int b) {
    vga_setpalette(i,r,g,b);
}


int
processUserInput(void) {
    static int buttons=0;
    int statusCheckCountdown = 1000;
    int quitrequested=0;
    int mousemoved=0;
    int oldMouseX = mouseX;
    int oldMouseY = mouseY;
    int oldbuttons = buttons;

    mouse_update();
    mouseX = mouse_getx();
    mouseY = mouse_gety();
    buttons=mouse_getbutton();

    if (mouseX != oldMouseX || mouseY != oldMouseY)
        mousemoved=1;

    if (mouseX < 0) mouseX = 0; else if (mouseX > width-12) mouseX = width-12;
    if (mouseY < 0) mouseY = 0; else if (mouseY > height-12) mouseY = height-12;
	
    if (mousemoved) {
        symbol = (mouseX-SYMBOLOFFSET)/SYMBOLSPACING;
        if (mouseY > 32 || mouseX < SYMBOLOFFSET) symbol = -1;
        fadeCount = 200;
	    
        if (showTracks) {
            if (mouseX > 40)
                showTracks = 0;
        } else if ((source & SYNSND_CD)
                   && mouseX <= 40 && mouseY <= 32) {
            showTracks = 1;
            trackCount = cdGetTrackCount();
            if (trackCount)
                trackStep = (200-65)/trackCount;
            else
                showTracks = 0;
        }
    } else 
        if (fadeCount > 0) fadeCount--;
	
    if ((source & SYNSND_CD)
        && ++statusCheckCountdown > (fadeCount ? 100 : 1000)) {
        statusCheckCountdown = 0;
        cdGetStatus(&track,&paused,&playing);
	    
        if (!paused && !playing) { /* Loop when finished, auto play on start */
            cdPlay(1); /* Allows you to leave it running */
            /*            unattended (eg as a demo) */
            cdGetStatus(&track,&paused,&playing);
            if (!playing) fadeCount = 10;
        }
    }
	
    if (buttons && !oldbuttons) {
        if (showTracks) {
            int which = (mouseY-32)/trackStep+1;
            if (which > 0 && which <= trackCount) {
                cdPlay(which);
                showTracks = 0;
            }
        }
	    
        fadeCount = 20;
	    
        if (source & SYNSND_CD) {
            cdGetStatus(&track,&paused,&playing);
        }
        switch(symbol) {
        case 0 :
            cdPlay(track>1?track-1:1);
            break;
        case 1 :
            if (playing)
                cdPlay(track);
            else
                if (paused)
                    cdResume();
                else cdPlay(1);
            break;
        case 2 :
            cdPause();
            break;
        case 3 :
            cdPlay(track<1?1:track+1);
            break;
        case 4 : quitrequested=1;
        }
        if (source & SYNSND_CD) {
            cdGetStatus(&track,&paused,&playing);
        }
    }
    
    /* Stop if quit was requested */
    if (quitrequested) {
        return 1;
    }
    return 0;
}


void
showOutput(void) {
    int symbols[SYMBOLCOUNT] = {12, 10, 13, 11, 14};
    int i=SYMBOLCOUNT-1;

    if (fadeCount) {
        if (source & SYNSND_CD) {
            int x,y,numberColor=0,number=track;
            if (showTracks) {
                for(i=0;i<trackCount;i++)
                    for(y=32+i*trackStep;y<32+(i*2+1)*trackStep/2+1;y++)
                        for(x=10;x<40;x++) {
                            int o = x+y*width,
                                c = (127+(int)(x+15+(sin(x/4.0+y/16.0)*5))%20*(128/20)+
                                     +127+(int)(x+0.5+15+(sin((x+0.5)/4.0+y/16.0)*5))%20*(128/20)) >> 1;
                            if (mouseY > 32 && (mouseY-32)/trackStep == i) backbuf[o+1] = c;
                            else {
                                backbuf[o] = c;
                                if (track == i+1) backbuf[o+1] = c;
                            }
                        }
                i = (mouseY-32)/trackStep+1;
                if (mouseY > 32 && i > 0 && i <= trackCount) {
                    number = i;
                    numberColor = 1;
                }
            }
            if (number >= 10) putIcon(0,0,number/10%10,numberColor);
            putIcon(20,0,number%10,numberColor);
            
            for(i=0;i<SYMBOLCOUNT-1;i++)
                putIcon(SYMBOLOFFSET+i*SYMBOLSPACING,0,symbols[i],(symbol==i?1:0));
            if (playing)
                putIcon(SYMBOLOFFSET+1*SYMBOLSPACING,0,symbols[1],1);
            else
                putIcon(SYMBOLOFFSET+2*SYMBOLSPACING,0,symbols[2],1);
        }
        putIcon(SYMBOLOFFSET+i*SYMBOLSPACING,0,symbols[i],(symbol==i?1:0));
        putIcon(mouseX,mouseY,15,0);
        putIcon(mouseX,mouseY,16,1);
    }
    
    gfxUpdate();
}
